// Copyright Epic Games, Inc. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "InputActionValue.h"
#include "ItemActor.h"
#include "Components/BoxComponent.h"
#include "GameFramework/Character.h"
#include "SeagullGameCharacter.generated.h"

UCLASS(Blueprintable)
class ASeagullGameCharacter : public ACharacter
{
	GENERATED_BODY()

public:
	DECLARE_DYNAMIC_MULTICAST_DELEGATE(FOnPlayerDeath);

	DECLARE_DYNAMIC_MULTICAST_DELEGATE(FOnPlayerDamage);

	DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnGameEnd, FString, Message);

	UPROPERTY(BlueprintAssignable, Category = "Events")
	FOnPlayerDeath OnPlayerDamage;
	UPROPERTY(BlueprintAssignable, Category = "Events")
	FOnPlayerDeath OnPlayerDeath;
	UPROPERTY(BlueprintAssignable, Category = "Events")
	FOnGameEnd OnGameEnd;

	ASeagullGameCharacter();

	// Called every frame.
	virtual void Tick(float DeltaSeconds) override;

	/** Returns TopDownCameraComponent sub-object **/
	FORCEINLINE class UCameraComponent* GetTopDownCameraComponent() const { return TopDownCameraComponent; }
	/** Returns CameraBoom sub-object **/
	FORCEINLINE class USpringArmComponent* GetCameraBoom() const { return CameraBoom; }
	/** MappingContext */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = Input, meta = (AllowPrivateAccess = "true"))
	class UInputMappingContext* DefaultMappingContext;

	/** Jump Input Action */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = Input, meta = (AllowPrivateAccess = "true"))
	class UInputAction* JumpAction;

	/** Pickup Input Action */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = Input, meta = (AllowPrivateAccess = "true"))
	class UInputAction* PickupAction;

	/** Move Input Action */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = Input, meta = (AllowPrivateAccess = "true"))
	class UInputAction* MoveAction;

	/** Look Input Action */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = Input, meta = (AllowPrivateAccess = "true"))
	class UInputAction* LookAction;

	virtual void BeginPlay() override;

	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "Health")
	float MaxHealth = 100;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "Health")
	float CurrentHealth = 100;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "Health")
	float DefaultHealth = 100;

	UFUNCTION(BlueprintCallable, Category = "Health")
	void DamagePlayer(float DamageAmount);
	UFUNCTION(BlueprintCallable, Category = "Health")
	void HealPlayer(float HealAmount);

	UFUNCTION(Exec, BlueprintCallable, Category = "General")
	void StartGame();
	UFUNCTION(BlueprintCallable, Category = "General")
	void EndGame();
	UPROPERTY(BlueprintReadOnly, Category = "General")
	FTimerHandle GameTimerHandle;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "General")
	float GameTime = 120;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "General")
	float Score = 0;

	UFUNCTION(BlueprintCallable, Category = "Item")
	void PickupItem();
	UFUNCTION(BlueprintCallable, Category = "Item")
	void DropItem();
	UFUNCTION(BlueprintCallable, Category = "Item")
	void OnPickupBoxBeginOverlap(UPrimitiveComponent* OverlappedComp, AActor* OtherActor, UPrimitiveComponent* OtherComp,
	                             int32 OtherBodyIndex, bool bFromSweep, const FHitResult& SweepResult);
	UFUNCTION(BlueprintCallable, Category = "Item")
	void OnPickupBoxEndOverlap(UPrimitiveComponent* OverlappedComp, AActor* OtherActor, UPrimitiveComponent* OtherComp,
	                           int32 OtherBodyIndex);
	UPROPERTY(BlueprintReadOnly, Category = "Item")
	AActor* CurrentItem;
	UPROPERTY(BlueprintReadOnly, Category = "Item")
	AActor* ClosestItemActor;
	UPROPERTY(BlueprintReadOnly, Category = "Item")
	UBoxComponent* PickupBox;

	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "John")
	float JohnsCurrentHunger = 0;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "John")
	float JohnsDefaultHunger = 100;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "John")
	float JohnsMaxHunger = 200;
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "John")
	float JohnsHungerDeclineAmount = 10.f;
	UFUNCTION(BlueprintCallable, Category = "John")
	void IncreaseJohnsHunger(float HungerAmount);

protected:
	// APawn interface
	virtual void SetupPlayerInputComponent(class UInputComponent* PlayerInputComponent) override;
	/** Called for movement input */
	void Move(const FInputActionValue& Value);

	/** Called for looking input */
	void Look(const FInputActionValue& Value);

private:
	/** Top down camera */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = Camera, meta = (AllowPrivateAccess = "true"))
	class UCameraComponent* TopDownCameraComponent;

	/** Camera boom positioning the camera above the character */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = Camera, meta = (AllowPrivateAccess = "true"))
	class USpringArmComponent* CameraBoom;

	bool bHasGameEnded = false;
};
